==> 1.0.0.tar.gz: lib/singly_linked_list.dart <==
// Copyright 2020 Ammar Yasser. All rights reserved.
// Use of this source code is governed by a BSD 2-Clause "Simplified" license
// that can be found in the LICENSE file.

library linear_structures;

/// Complexity of each operation
/// ----------------------------
///  operation   | complexity
///  peekFirst   |   O(1)
///  peekLast    |   O(1)
///  isEmpty     |   O(1)
///  isNotEmpty  |   O(1)
///  length      |   O(1)
///  contains    |   O(n)
///  peek        |   O(n)
///  getAll      |   O(n)
///  addFirst    |   O(1)
///  addLast     |   O(1)
///  remove      |   O(n)
///  removeFirst |   O(1)
///  removeLast  |   O(n)
///  clear       |   O(1)

/// Implementation of Singly LinkedList
class SinglyLinkedList<T> {
  int _size = 0;
  _Node<T> _head;
  _Node<T> _tail;

  /// Returns the first node data.
  T get peekFirst => _head?.data;

  /// Returns the last node data.
  T get peekLast => _tail?.data;

  /// Returns true if there are no nodes in this list.
  bool get isEmpty => _head == null;

  /// Returns true if there is at least one node in this list.
  bool get isNotEmpty => _head != null;

  /// Returns the number of nodes in this list.
  int get length => _size;

  /// Returns true if this list contains the specified node data.
  bool contains(T obj) {
    if (_head == null) return false;
    _Node<T> last = _head;
    while (last.next != null) {
      if (last.data == obj) return true;
      last = last.next;
    }
    return false;
  }

  /// Returns the node data at the specified position.
  /// This method returns null if the specified position contains nothing
  /// or the list is empty.
  T peek(int index) {
    if (index + 1 > _size || index < 0) return null;
    _Node<T> last = _head;
    for (int i = 0; i < index; i++) last = last.next;
    return last.data;
  }

  /// Gets all nodes data in this LinkedList and then returns them as type List.
  /// This method returns null if the list is empty.
  List<T> getAll() {
    if (_head == null) return null;
    if (_head == _tail) return [_head.data];
    List<T> nodes = List<T>();
    _Node<T> last = _head;
    while (last != null) {
      nodes.add(last.data);
      last = last.next;
    }
    return nodes;
  }

  /// Adds a node to the beginning of the list.
  void addFirst(T obj) {
    _Node<T> newNode = _Node(obj);
    if (_head == null) {
      _head = _tail = newNode;
    } else {
      newNode.next = _head;
      _head = newNode;
    }
    _size++;
  }

  /// Adds a node to the end of the list.
  void addLast(T obj) {
    _Node<T> newNode = _Node(obj);
    if (_head == null) {
      _head = _tail = newNode;
    } else {
      _tail.next = newNode;
      _tail = newNode;
    }
    _size++;
  }

  /// Removes a specific node from this list and then returns its data.
  /// This method returns null if the list is empty or the specific node
  /// does not exists.
  T remove(T obj) {
    if (_head == null) return null;
    _Node<T> current = _head, previous;
    while (current != null) {
      if (current.data == obj) {
        if (current == _head) return removeFirst();
        previous.next = current.next;
        _size--;
        return current.data;
      }
      previous = current;
      current = current.next;
    }
    return null;
  }

  /// Removes and returns the first node data from this list.
  /// This method returns null if the list is empty.
  T removeFirst() {
    if (_head == null) return null;
    T removedData = _head.data;
    if (_head == _tail) {
      _head = _tail = null;
    } else {
      _head = _head.next;
    }
    _size--;
    return removedData;
  }

  /// Removes and returns the last node data from this list.
  /// This method returns null if the list is empty.
  T removeLast() {
    if (_head == null) return null;
    _Node<T> current = _head, previous;
    if (_head == _tail) {
      _head = _tail = null;
    } else {
      while (current != _tail) {
        previous = current;
        current = current.next;
      }
      previous.next = null;
      _tail = previous;
    }
    _size--;
    return current.data;
  }

  /// Removes all nodes from this list.
  void clear() {
    _size = 0;
    _head = null;
    _tail = null;
  }
}

class _Node<T> {
  T data;
  _Node<T> next;

  _Node(T obj) {
    data = obj;
    next = null;
  }
}

==> 1.0.1.tar.gz: lib/singly_linked_list.dart <==
// Copyright 2020 Ammar Yasser. All rights reserved.
// Use of this source code is governed by a BSD 2-Clause "Simplified" license
// that can be found in the LICENSE file.

library linear_structures;

/// Complexity of each operation
/// ----------------------------
///  operation   | complexity
///  peekFirst   |   O(1)
///  peekLast    |   O(1)
///  isEmpty     |   O(1)
///  isNotEmpty  |   O(1)
///  length      |   O(1)
///  contains    |   O(n)
///  peek        |   O(n)
///  getAll      |   O(n)
///  addFirst    |   O(1)
///  addLast     |   O(1)
///  remove      |   O(n)
///  removeFirst |   O(1)
///  removeLast  |   O(n)
///  clear       |   O(1)

/// Implementation of Singly LinkedList
class SinglyLinkedList<T> {
  int _size = 0;
  _Node<T> _head;
  _Node<T> _tail;

  /// Returns the first node data.
  T get peekFirst => _head?.data;

  /// Returns the last node data.
  T get peekLast => _tail?.data;

  /// Returns true if there are no nodes in this list.
  bool get isEmpty => _head == null;

  /// Returns true if there is at least one node in this list.
  bool get isNotEmpty => _head != null;

  /// Returns the number of nodes in this list.
  int get length => _size;

  /// Returns true if this list contains the specified node data.
  bool contains(T obj) {
    if (_head == null) return false;
    _Node<T> last = _head;
    while (last.next != null) {
      if (last.data == obj) return true;
      last = last.next;
    }
    return false;
  }

  /// Returns the node data at the specified position.
  /// This method returns null if the specified position contains nothing
  /// or the list is empty.
  T peek(int index) {
    if (index + 1 > _size || index < 0) return null;
    _Node<T> last = _head;
    for (int i = 0; i < index; i++) {
      last = last.next;
    }
    return last.data;
  }

  /// Gets all nodes data in this LinkedList and then returns them as type List.
  /// This method returns null if the list is empty.
  List<T> getAll() {
    if (_head == null) return null;
    if (_head == _tail) return [_head.data];
    List<T> nodes = List<T>();
    _Node<T> last = _head;
    while (last != null) {
      nodes.add(last.data);
      last = last.next;
    }
    return nodes;
  }

  /// Adds a node to the beginning of the list.
  void addFirst(T obj) {
    _Node<T> newNode = _Node(obj);
    if (_head == null) {
      _head = _tail = newNode;
    } else {
      newNode.next = _head;
      _head = newNode;
    }
    _size++;
  }

  /// Adds a node to the end of the list.
  void addLast(T obj) {
    _Node<T> newNode = _Node(obj);
    if (_head == null) {
      _head = _tail = newNode;
    } else {
      _tail.next = newNode;
      _tail = newNode;
    }
    _size++;
  }

  /// Removes a specific node from this list and then returns its data.
  /// This method returns null if the list is empty or the specific node
  /// does not exists.
  T remove(T obj) {
    if (_head == null) return null;
    _Node<T> current = _head, previous;
    while (current != null) {
      if (current.data == obj) {
        if (current == _head) return removeFirst();
        previous.next = current.next;
        _size--;
        return current.data;
      }
      previous = current;
      current = current.next;
    }
    return null;
  }

  /// Removes and returns the first node data from this list.
  /// This method returns null if the list is empty.
  T removeFirst() {
    if (_head == null) return null;
    T removedData = _head.data;
    if (_head == _tail) {
      _head = _tail = null;
    } else {
      _head = _head.next;
    }
    _size--;
    return removedData;
  }

  /// Removes and returns the last node data from this list.
  /// This method returns null if the list is empty.
  T removeLast() {
    if (_head == null) return null;
    _Node<T> current = _head, previous;
    if (_head == _tail) {
      _head = _tail = null;
    } else {
      while (current != _tail) {
        previous = current;
        current = current.next;
      }
      previous.next = null;
      _tail = previous;
    }
    _size--;
    return current.data;
  }

  /// Removes all nodes from this list.
  void clear() {
    _size = 0;
    _head = null;
    _tail = null;
  }
}

class _Node<T> {
  T data;
  _Node<T> next;

  _Node(T obj) {
    data = obj;
    next = null;
  }
}
